package database

import (
	"fmt"

	"git.jdev.run/pkg/config"

	"gorm.io/gorm"

	// Drivers
	"gorm.io/driver/mysql"
	"gorm.io/driver/postgres"
	// "gorm.io/driver/sqlite"
	// Sqlite driver based on GGO
)

var (
	DB *gorm.DB
)

// NewDatabase returns a gorm.DB struct, gorm.DB.DB() returns a database handle
// see http://golang.org/pkg/database/sql/#DB
func New() (*gorm.DB, error) {
	// Postgres

	var cnf config.Config = config.GetGlobalConfig()

	if cnf.DatabaseEngine == "postgres" {
		// Connection args
		// see https://godoc.org/github.com/lib/pq#hdr-Connection_String_Parameters
		dsn := fmt.Sprintf(
			"sslmode=disable host=%s port=%d user=%s password='%s' dbname=%s",
			cnf.DatabaseHost,
			cnf.DatabasePort,
			cnf.DatabaseUser,
			cnf.DatabasePassword,
			cnf.DatabaseName,
		)

		db, err := gorm.Open(postgres.Open(dsn), &gorm.Config{})
		if err != nil {
			return db, err
		}

		d, _ := db.DB()
		// Max idle connections
		d.SetMaxIdleConns(cnf.DatabaseMaxIdleConns)

		// Max open connections
		d.SetMaxOpenConns(cnf.DatabaseMaxOpenConns)

		// Database logging
		// db.LogMode(config.IsDevelopment())

		return db, nil
	}
	// if cnf.DatabaseEngine == "sqlite" {
	// 	// Connection args
	// 	// see https://godoc.org/github.com/lib/pq#hdr-Connection_String_Parameters

	// 	db, err := gorm.Open(sqlite.Open(cnf.DatabaseName), &gorm.Config{})
	// 	if err != nil {
	// 		return db, err
	// 	}

	// 	return db, nil
	// }
	if cnf.DatabaseEngine == "mysql" {
		// Connection args
		// see https://godoc.org/github.com/lib/pq#hdr-Connection_String_Parameters

		dsn := fmt.Sprintf(
			"%s:%s@tcp(%s:%d)/%s?charset=utf8mb4&parseTime=True&loc=Local",
			cnf.DatabaseUser,
			cnf.DatabasePassword,
			cnf.DatabaseHost,
			cnf.DatabasePort,
			cnf.DatabaseName,
		)

		db, err := gorm.Open(mysql.Open(dsn), &gorm.Config{})
		if err != nil {
			return db, err
		}

		return db, nil
	}

	// Database type not supported
	return nil, fmt.Errorf("database type %s not suppported", cnf.DatabaseEngine)
}

func GetDatastore() (*gorm.DB, error) {
	if DB != nil {
		return DB, nil
	}
	d, err := New()
	if err != nil {
		return d, err
	}
	DB = d
	return DB, nil
}
